/*********************************************************************
 * @file	rtos_bluetooth.h
 * @version 	1.0
 * @date    	07. February 2020
 * @brief
 *********************************************************************
 * @attention
 *
 * TODO
 *
 ********************************************************************/

#ifndef RTOS_BLUETOOTH_H
#define RTOS_BLUETOOTH_H

#ifdef APP_NCP
#include "rtosncp_gecko.h"
#else
#include "rtos_gecko.h"
#endif
#include "RTOS.h"

/* Bluetooth event */
extern OS_EVENT bluetoothOSEvent;

/* Bluetooth event Bit-Mask definition */
#define BLUETOOTH_EVENT_BITMASK_STACK       ((OS_U32)1)     // Bluetooth task needs an update
#define BLUETOOTH_EVENT_BITMASK_LL          ((OS_U32)2)     // LinkLayer task needs an update
#define BLUETOOTH_EVENT_BITMASK_CMD_WAITING ((OS_U32)4)     // BGAPI command is waiting to be processed
#define BLUETOOTH_EVENT_BITMASK_RSP_WAITING ((OS_U32)8)     // BGAPI response is waiting to be processed
#define BLUETOOTH_EVENT_BITMASK_EVT_WAITING ((OS_U32)16)    // BGAPI event is waiting to be processed
#define BLUETOOTH_EVENT_BITMASK_EVT_HANDLED ((OS_U32)32)    // BGAPI event is handled

/* Bluetooth event data pointer */
extern volatile struct gecko_cmd_packet* bluetoothEvent;

/* Function prototype for initializing Bluetooth stack */
typedef errorcode_t(*bluetoothStackInitFunc)();

/*
 * Start Bluetooth tasks. The given Bluetooth stack initialization function
 * will be called at a proper time. Application should not initialize
 * Bluetooth stack anywhere else.
 *
 * @param ll_priority link layer task priority
 * @param stack_priority Bluetooth stack task priority
 * @param initialize_bluetooth_stack The function for initializing Bluetooth stack
 */
void bluetoothStart(OS_PRIO linkLayerPriority,
                    OS_PRIO stackPriority,
                    bluetoothStackInitFunc initializeBluetoothStack);

/*
 * Set the callback for wakeup, Bluetooth task will call this when it has a new event
 * It must only be used to wake up application task, for example by posting task semaphore
 */
typedef void (*wakeupCallback)();
void bluetoothSetWakeupCallback(wakeupCallback cb); 

/* Bluetooth stack needs an update */
void bluetoothUpdate();

/* LinkLayer is updated */
void bluetoothLinkLayerCallback();

/* Mutex functions for using Bluetooth from multiple tasks */
void bluetoothLockMutex(OS_U8* mutexStatus);
void bluetoothUnlockMutex(void);

#endif //RTOS_BLUETOOTH_H
